<?php

namespace Drupal\Tests\mustache\Kernel;

use Drupal\KernelTests\KernelTestBase;
use Drupal\mustache\Helpers\MustacheRenderTemplate;
use Drupal\user\Entity\User;

/**
 * Kernel tests for inline templates.
 *
 * @group mustache
 */
class InlineTemplatesTest extends KernelTestBase {

  protected static $modules = [
    'system',
    'user',
    'field',
    'mustache',
  ];

  /**
   * {@inheritdoc}
   */
  public function setUp(): void {
    parent::setUp();
    $this->installEntitySchema('user');
    $this->installConfig(static::$modules);
    User::create(['uid' => 1, 'name' => 'admin'])->save();
  }

  /**
   * Tests rendering of inline templates.
   */
  public function testRenderInline() {
    /** @var \Drupal\Core\Render\RendererInterface $renderer */
    $renderer = \Drupal::service('renderer');
    /** @var \Drupal\Core\Session\AccountSwitcherInterface $account_switcher */
    $account_switcher = \Drupal::service('account_switcher');

    $render_array = MustacheRenderTemplate::build('my_inline_template', 'Value: {{ baz.qux }}')
      ->usingData(['foo' => ['bar' => ['baz' => ['qux' => 'Hello!']]]])
      ->selectingSubsetFromData(['foo', 'bar'])
      ->toRenderArray();
    $rendered = trim((string) $renderer->renderPlain($render_array));
    $this->assertEquals('Value: Hello!', $rendered);

    $admin_user = User::load(1);
    $render_array = MustacheRenderTemplate::build('my_inline_template', 'User name: {{ user.account-name }}')
      ->withTokens(['user' => $admin_user])
      ->toRenderArray();
    $rendered = trim((string) $renderer->renderPlain($render_array));
    $this->assertEquals('User name:', $rendered, 'User without permission is not allowed to view Token data.');
    $render_array = MustacheRenderTemplate::build('my_inline_template', '{{#user}}User name: {{ user.account-name }}{{/user}}')
      ->withTokens(['user' => $admin_user])
      ->toRenderArray();
    $rendered = trim((string) $renderer->renderPlain($render_array));
    $this->assertEquals('', $rendered, 'User without permission is not allowed to view Token data.');
    $render_array = MustacheRenderTemplate::build('my_inline_template', '{{#user.uid}}User name: {{ user.account-name }}{{/user.uid}}')
      ->withTokens(['user' => $admin_user])
      ->toRenderArray();
    $rendered = trim((string) $renderer->renderPlain($render_array));
    $this->assertEquals('', $rendered, 'User without permission is not allowed to view Token data.');
    $account_switcher->switchTo($admin_user);
    $render_array = MustacheRenderTemplate::build('my_inline_template', '{{#user}}User name: {{ user.account-name }}{{/user}}')
      ->withTokens(['user' => $admin_user])
      ->toRenderArray();
    $rendered = trim((string) $renderer->renderPlain($render_array));
    $this->assertEquals('User name: admin', $rendered, 'User with permission is allowed to view Token data.');
    $render_array = MustacheRenderTemplate::build('my_inline_template', '{{#user.account-name}}User name: {{ user.account-name }}{{/user.account-name}}')
    ->withTokens(['user' => $admin_user])
    ->toRenderArray();
    $rendered = trim((string) $renderer->renderPlain($render_array));
    $this->assertEquals('User name: admin', $rendered, 'User with permission is allowed to view Token data.');
    $render_array = MustacheRenderTemplate::build('my_inline_template', '{{#user.uid}}User name: {{ user.account-name }}{{/user.uid}}')
    ->withTokens(['user' => $admin_user])
    ->toRenderArray();
    $rendered = trim((string) $renderer->renderPlain($render_array));
    $this->assertEquals('User name: admin', $rendered, 'User with permission is allowed to view Token data.');
    $account_switcher->switchBack();
  }

  /**
   * Tests magic conditions using inline templates.
   */
  public function testMagicConditions() {
    $admin_user = User::load(1);
    /** @var \Drupal\Core\Render\RendererInterface $renderer */
    $renderer = \Drupal::service('renderer');
    /** @var \Drupal\Core\Session\AccountSwitcherInterface $account_switcher */
    $account_switcher = \Drupal::service('account_switcher');
    $account_switcher->switchTo($admin_user);

    $render_array = MustacheRenderTemplate::build('my_inline_template', '{{#if.foo.bar.empty}}This SHOULD show up!{{/if.foo.bar.empty}}{{^if.foo.bar.empty}}This should not show up!{{/if.foo.bar.empty}}')
      ->toRenderArray();
    $rendered = trim((string) $renderer->renderPlain($render_array));
    $this->assertEquals('This SHOULD show up!', $rendered);
    $render_array = MustacheRenderTemplate::build('my_inline_template', '{{#if.foo.bar.empty}}This SHOULD show up!{{/if.foo.bar.empty}}{{^if.foo.bar.empty}}This should not show up!{{/if.foo.bar.empty}}')
      ->usingData(['foo' => ['bar' => []]])
      ->toRenderArray();
    $rendered = trim((string) $renderer->renderPlain($render_array));
    $this->assertEquals('This SHOULD show up!', $rendered);
    $render_array = MustacheRenderTemplate::build('my_inline_template', '{{#if.foo.bar.empty}}This SHOULD show up!{{/if.foo.bar.empty}}{{^if.foo.bar.empty}}This should not show up!{{/if.foo.bar.empty}}')
      ->usingData(['foo' => ['bar' => '']])
      ->toRenderArray();
    $rendered = trim((string) $renderer->renderPlain($render_array));
    $this->assertEquals('This SHOULD show up!', $rendered);
    $render_array = MustacheRenderTemplate::build('my_inline_template', '{{#if.foo.bar.empty}}This SHOULD show up!{{/if.foo.bar.empty}}{{^if.foo.bar.empty}}This should not show up!{{/if.foo.bar.empty}}')
      ->usingData(['foo' => ['bar' => 0]])
      ->toRenderArray();
    $rendered = trim((string) $renderer->renderPlain($render_array));
    $this->assertEquals('This SHOULD show up!', $rendered);
    $render_array = MustacheRenderTemplate::build('my_inline_template', '{{#if.foo.bar.numeric}}Is numeric!{{/if.foo.bar.numeric}}{{^if.foo.bar.numeric}}Is not numeric!{{/if.foo.bar.numeric}}')
      ->usingData(['foo' => ['bar' => 0]])
      ->toRenderArray();
    $rendered = trim((string) $renderer->renderPlain($render_array));
    $this->assertEquals('Is numeric!', $rendered);
    $render_array = MustacheRenderTemplate::build('my_inline_template', '{{#if.foo.bar.empty}}This should not show up!{{/if.foo.bar.empty}}{{^if.foo.bar.empty}}This SHOULD should show up :){{/if.foo.bar.empty}}')
      ->usingData(['foo' => ['bar' => 1]])
      ->toRenderArray();
    $rendered = trim((string) $renderer->renderPlain($render_array));
    $this->assertEquals('This SHOULD should show up :)', $rendered);
    $render_array = MustacheRenderTemplate::build('my_inline_template', '{{#if.foo.bar.numeric}}Is numeric!{{/if.foo.bar.numeric}}{{^if.foo.bar.numeric}}Is not numeric!{{/if.foo.bar.numeric}}')
      ->usingData(['foo' => ['bar' => 0]])
      ->toRenderArray();
    $rendered = trim((string) $renderer->renderPlain($render_array));
    $this->assertEquals('Is numeric!', $rendered);
    $render_array = MustacheRenderTemplate::build('my_inline_template', '{{#if.foo.bar.empty}}This should not show up!{{/if.foo.bar.empty}}{{^if.foo.bar.empty}}This SHOULD should show up :){{/if.foo.bar.empty}}')
      ->usingData(['foo' => ['bar' => ['baz' => 'qux']]])
      ->toRenderArray();
    $rendered = trim((string) $renderer->renderPlain($render_array));
    $this->assertEquals('This SHOULD should show up :)', $rendered);

    $account_switcher->switchBack();
  }

}
